#/usr/bin/env python3

import markdown
import xml.etree.ElementTree as etree
from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor, SimpleTagInlineProcessor
from markdown.preprocessors import Preprocessor


class MetaStripper(Extension):
    """
    An extension that strips the metadata off the front of Obsidian pages, as it's already been parsed in an
    earlier step.
    """

    class MetaStripperProc(Preprocessor):
        def run(self, lines):
            if lines[0] == '---':
                lines.pop(0)
                while lines[0] != '---':
                    lines.pop(0)
                lines.pop(0)
            return lines

    def extendMarkdown(self, md):
        md.preprocessors.register(MetaStripper.MetaStripperProc(md), 'metastripper', 0)


class ObsidianLinks(Extension):
    """An extension that processes Obsidian internal links in the [[page name]] format."""
    def __init__(self, context, **kwargs):
        super(ObsidianLinks, self).__init__(**kwargs)
        self._context = context

    class ObsidianLinksProc(InlineProcessor):
        def __init__(self, pattern, md, context):
            super(ObsidianLinks.ObsidianLinksProc, self).__init__(pattern, md)
            self._context = context

        def parse_reference(self, contents):
            text = None
            t = contents.split('|')
            if len(t) > 1:
                text = t[1]
                contents = t[0]

            node, linktype = self._context.src_index.lookup(contents)
            if not text:
                text = contents
            if node:
                return node.link_target(self._context.url_prefix), text
            return None, text

        def handleMatch(self, m, data):
            link, text = self.parse_reference(m.group(0)[2:-2])
            if link is None:
                el = etree.Element('span')
                el.set('class', 'invalid-reference')
                el.text = text
            else:
                el = etree.Element('a')
                el.set('href', link)
                el.text = text
            return el, m.start(0), m.end(0)

    def extendMarkdown(self, md):
        OBSLINK_PATTERN = r'\[\[(.*?)\]\]'
        md.inlinePatterns.register(ObsidianLinks.ObsidianLinksProc(OBSLINK_PATTERN, md, self._context),
                                   'obsidian_links', 0)


class ObsidianInlines(Extension):
    """An extension that handles the special Obsidian markdown format sequences."""
    def extendMarkdown(self, md):
        md.inlinePatterns.register(SimpleTagInlineProcessor(r'()~~(.*?)~~', 'del'), 'strikeout', 0)
        md.inlinePatterns.register(SimpleTagInlineProcessor(r'()\=\=(.*?)\=\=', 'ins'), 'highlight', 0)


def create_markdown_parser(context):
    return markdown.Markdown(extensions=[MetaStripper(), ObsidianLinks(context), ObsidianInlines()])
